@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2021, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,6 +18,8 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
+ * SPDX-License-Identifier: curl
+ *
  ***************************************************************************/
 #include "server_setup.h"
 #include <stdlib.h>
@@ -60,7 +62,6 @@
 #include "curlx.h" /* from the private lib dir */
 #include "getpart.h"
 #include "inet_pton.h"
-#include "util.h"
 #include "server_sockaddr.h"
 #include "warnless.h"
 
@@ -111,13 +112,15 @@ struct configurable {
   int testnum;
 };
 
-#define REQUEST_DUMP  "log/server.input"
+#define REQUEST_DUMP  "server.input"
 #define CONFIG_VERSION 5
 
 static struct configurable config;
 
 const char *serverlogfile = DEFAULT_LOGFILE;
 static const char *configfile = DEFAULT_CONFIG;
+const char *logdir = "log";
+char loglockfile[256];
 
 #ifdef ENABLE_IPV6
 static bool use_ipv6 = FALSE;
@@ -503,14 +506,16 @@ static curl_socket_t mqttit(curl_socket_t fd)
   char client_id[MAX_CLIENT_ID_LENGTH];
   long testno;
   FILE *stream = NULL;
-
+  FILE *dump;
+  char dumpfile[256];
 
   static const char protocol[7] = {
     0x00, 0x04,       /* protocol length */
     'M','Q','T','T',  /* protocol name */
     0x04              /* protocol level */
   };
-  FILE *dump = fopen(REQUEST_DUMP, "ab");
+  msnprintf(dumpfile, sizeof(dumpfile), "%s/%s", logdir, REQUEST_DUMP);
+  dump = fopen(dumpfile, "ab");
   if(!dump)
     goto end;
 
@@ -641,7 +646,7 @@ static curl_socket_t mqttit(curl_socket_t fd)
       /* there's a QoS byte (two bits) after the topic */
 
       logmsg("SUBSCRIBE to '%s' [%d]", topic, packet_id);
-      stream = test2fopen(testno);
+      stream = test2fopen(testno, logdir);
       error = getpart(&data, &datalen, "reply", "data", stream);
       if(!error) {
         if(!config.publish_before_suback) {
@@ -771,9 +776,9 @@ static bool incoming(curl_socket_t listenfd)
       else {
         logmsg("====> Client connect, fd %d. Read config from %s",
                newfd, configfile);
-        set_advisor_read_lock(SERVERLOGS_LOCK);
+        set_advisor_read_lock(loglockfile);
         (void)mqttit(newfd); /* until done */
-        clear_advisor_read_lock(SERVERLOGS_LOCK);
+        clear_advisor_read_lock(loglockfile);
 
         logmsg("====> Client disconnect");
         sclose(newfd);
@@ -961,6 +966,11 @@ int main(int argc, char *argv[])
       if(argc>arg)
         serverlogfile = argv[arg++];
     }
+    else if(!strcmp("--logdir", argv[arg])) {
+      arg++;
+      if(argc>arg)
+        logdir = argv[arg++];
+    }
     else if(!strcmp("--ipv6", argv[arg])) {
 #ifdef ENABLE_IPV6
       ipv_inuse = "IPv6";
@@ -996,6 +1006,7 @@ int main(int argc, char *argv[])
            " --config [file]\n"
            " --version\n"
            " --logfile [file]\n"
+           " --logdir [directory]\n"
            " --pidfile [file]\n"
            " --portfile [file]\n"
            " --ipv4\n"
@@ -1005,6 +1016,9 @@ int main(int argc, char *argv[])
     }
   }
 
+  msnprintf(loglockfile, sizeof(loglockfile), "%s/%s",
+            logdir, SERVERLOGS_LOCK);
+
 #ifdef WIN32
   win32_init();
   atexit(win32_cleanup);
