@@ -15,7 +15,7 @@
  */
 
 /*
- * Copyright (C) 2005 - 2021, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) Daniel Stenberg, <daniel@haxx.se>, et al.
  * Copyright (c) 1983, Regents of the University of California.
  * All rights reserved.
  *
@@ -46,6 +46,8 @@
  * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
+ *
+ * SPDX-License-Identifier: BSD-4-Clause-UC
  */
 
 #include "server_setup.h"
@@ -86,6 +88,8 @@
 #include <pwd.h>
 #endif
 
+#include <ctype.h>
+
 #define ENABLE_CURLX_PRINTF
 /* make the curlx header define all printf() functions to use the curlx_*
    versions instead */
@@ -159,7 +163,7 @@ struct bf {
 #define DEFAULT_LOGFILE "log/tftpd.log"
 #endif
 
-#define REQUEST_DUMP  "log/server.input"
+#define REQUEST_DUMP  "server.input"
 
 #define DEFAULT_PORT 8999 /* UDP */
 
@@ -211,6 +215,8 @@ static bool use_ipv6 = FALSE;
 static const char *ipv_inuse = "IPv4";
 
 const  char *serverlogfile = DEFAULT_LOGFILE;
+const char *logdir = "log";
+char loglockfile[256];
 static const char *pidname = ".tftpd.pid";
 static const char *portname = NULL; /* none by default */
 static int serverlogslocked = 0;
@@ -296,7 +302,7 @@ static void timer(int signum)
     }
     if(serverlogslocked) {
       serverlogslocked = 0;
-      clear_advisor_read_lock(SERVERLOGS_LOCK);
+      clear_advisor_read_lock(loglockfile);
     }
     exit(1);
   }
@@ -454,7 +460,7 @@ static ssize_t write_behind(struct testcase *test, int convert)
 
   if(!test->ofile) {
     char outfile[256];
-    msnprintf(outfile, sizeof(outfile), "log/upload.%ld", test->testno);
+    msnprintf(outfile, sizeof(outfile), "%s/upload.%ld", logdir, test->testno);
 #ifdef WIN32
     test->ofile = open(outfile, O_CREAT|O_RDWR|O_BINARY, 0777);
 #else
@@ -591,6 +597,11 @@ int main(int argc, char **argv)
       if(argc>arg)
         serverlogfile = argv[arg++];
     }
+    else if(!strcmp("--logdir", argv[arg])) {
+      arg++;
+      if(argc>arg)
+        logdir = argv[arg++];
+    }
     else if(!strcmp("--ipv4", argv[arg])) {
 #ifdef ENABLE_IPV6
       ipv_inuse = "IPv4";
@@ -625,6 +636,7 @@ int main(int argc, char **argv)
       puts("Usage: tftpd [option]\n"
            " --version\n"
            " --logfile [file]\n"
+           " --logdir [directory]\n"
            " --pidfile [file]\n"
            " --portfile [file]\n"
            " --ipv4\n"
@@ -635,6 +647,9 @@ int main(int argc, char **argv)
     }
   }
 
+  msnprintf(loglockfile, sizeof(loglockfile), "%s/%s",
+            logdir, SERVERLOGS_LOCK);
+
 #ifdef WIN32
   win32_init();
   atexit(win32_cleanup);
@@ -775,7 +790,7 @@ int main(int argc, char **argv)
       break;
     }
 
-    set_advisor_read_lock(SERVERLOGS_LOCK);
+    set_advisor_read_lock(loglockfile);
     serverlogslocked = 1;
 
 #ifdef ENABLE_IPV6
@@ -829,7 +844,7 @@ int main(int argc, char **argv)
 
     if(serverlogslocked) {
       serverlogslocked = 0;
-      clear_advisor_read_lock(SERVERLOGS_LOCK);
+      clear_advisor_read_lock(loglockfile);
     }
 
     logmsg("end of one transfer");
@@ -857,7 +872,7 @@ tftpd_cleanup:
 
   if(serverlogslocked) {
     serverlogslocked = 0;
-    clear_advisor_read_lock(SERVERLOGS_LOCK);
+    clear_advisor_read_lock(loglockfile);
   }
 
   restore_signal_handlers(true);
@@ -891,13 +906,17 @@ static int do_tftp(struct testcase *test, struct tftphdr *tp, ssize_t size)
 #endif
   const char *option = "mode"; /* mode is implicit */
   int toggle = 1;
+  FILE *server;
+  char dumpfile[256];
+
+  msnprintf(dumpfile, sizeof(dumpfile), "%s/%s", logdir, REQUEST_DUMP);
 
   /* Open request dump file. */
-  FILE *server = fopen(REQUEST_DUMP, "ab");
+  server = fopen(dumpfile, "ab");
   if(!server) {
     int error = errno;
     logmsg("fopen() failed with error: %d %s", error, strerror(error));
-    logmsg("Error opening file: %s", REQUEST_DUMP);
+    logmsg("Error opening file: %s", dumpfile);
     return -1;
   }
 
@@ -1000,7 +1019,7 @@ static int parse_servercmd(struct testcase *req)
   FILE *stream;
   int error;
 
-  stream = test2fopen(req->testno);
+  stream = test2fopen(req->testno, logdir);
   if(!stream) {
     error = errno;
     logmsg("fopen() failed with error: %d %s", error, strerror(error));
@@ -1111,7 +1130,7 @@ static int validate_access(struct testcase *test,
 
     (void)parse_servercmd(test);
 
-    stream = test2fopen(testno);
+    stream = test2fopen(testno, logdir);
 
     if(0 != partno)
       msnprintf(partbuf, sizeof(partbuf), "data%ld", partno);
